! $Id: physiq.F 1565 2011-08-31 12:53:29Z jghattas $
!#define IO_DEBUG

      SUBROUTINE phys1dAglae(nlon,nlev, &
     &            debut,lafin,pdtphys, &
     &            paprs,pplay,pphi,pphis,presnivs,clesphy0, &
     &            vitu,vitv,temp,qx, &
     &            flxmass_w, &
     &            d_u, d_v, d_t, d_qx, d_ps &
     &            , dudyn &
     &            , PVteta)

      USE dimphy
      USE infotrac
      USE geometry_mod, ONLY: cell_area, latitude_deg, longitude_deg
      USE phys_cal_mod, only: year_len, mth_len, days_elapsed, jh_1jan, &
         year_cur, mth_cur,jD_cur,jH_cur, jD_ref
      USE ioipsl , ONLY : getin

      IMPLICIT NONE
!======================================================================
! Objet: Moniteur general de la physique du modele
!======================================================================
!
!  Arguments:
!
! nlon----input-I-nombre de points horizontaux
! nlev----input-I-nombre de couches verticales, doit etre egale a klev
! debut---input-L-variable logique indiquant le premier passage
! lafin---input-L-variable logique indiquant le dernier passage
! jD_cur       -R-jour courant a l'appel de la physique (jour julien)
! jH_cur       -R-heure courante a l'appel de la physique (jour julien)
! pdtphys-input-R-pas d'integration pour la physique (seconde)
! paprs---input-R-pression pour chaque inter-couche (en Pa)
! pplay---input-R-pression pour le mileu de chaque couche (en Pa)
! pphi----input-R-geopotentiel de chaque couche (g z) (reference sol)
! pphis---input-R-geopotentiel du sol
! presnivs-input_R_pressions approximat. des milieux couches ( en PA)
! vitu-------input-R-vitesse dans la direction X (de O a E) en m/s
! vitv-------input-R-vitesse Y (de S a N) en m/s
! temp-------input-R-temperature (K)
! qx------input-R-humidite specifique (kg/kg) et d'autres traceurs
! d_t_dyn-input-R-tendance dynamique pour "t" (K/s)
! d_q_dyn-input-R-tendance dynamique pour "q" (kg/kg/s)
! flxmass_w -input-R- flux de masse verticale
! d_u-----output-R-tendance physique de "u" (m/s/s)
! d_v-----output-R-tendance physique de "v" (m/s/s)
! d_t-----output-R-tendance physique de "t" (K/s)
! d_qx----output-R-tendance physique de "qx" (kg/kg/s)
! d_ps----output-R-tendance physique de la pression au sol
!IM
! PVteta--output-R-vorticite potentielle a des thetas constantes
!======================================================================

!============================================================================
! 0/ Declarations
!============================================================================

! include
#include "dimensions.h"
#include "YOMCST.h"
#include "flux_arp.h"


      INTEGER jjmp1
      parameter (jjmp1=jjm+1-1/jjm)
      INTEGER iip1
      parameter (iip1=iim+1)

      INTEGER ivap          ! indice de traceurs pour vapeur d'eau
      PARAMETER (ivap=1)
      INTEGER iliq          ! indice de traceurs pour eau liquide
      PARAMETER (iliq=2)

! Variables argument:
      INTEGER nlon
      INTEGER nlev

      REAL pdtphys
      LOGICAL debut, lafin
      REAL paprs(klev+1)
      REAL pplay(klev)
      REAL pphi(klev)
      REAL pphis
      REAL presnivs(klev)
      REAL znivsig(klev)

      REAL vitu(klev)
      REAL vitv(klev)
      REAL temp(klev),theta(klev)
      REAL qx(klev,nqtot)
      REAL flxmass_w(klev)
      REAL omega(klev) ! vitesse verticale en Pa/s
      REAL d_u(klev)
      REAL d_v(klev)
      REAL d_t(klev)
      REAL d_qx(klev,nqtot)
      REAL d_ps
      REAL da(klev),phi(klev,klev),mp(klev)
      INTEGER nbteta
      PARAMETER(nbteta=3)
      REAL PVteta(nbteta)
      REAL ug, vg

      REAL dudyn(iip1,jjmp1,klev)
      REAL, DIMENSION(klev) :: rho,rhodz,ovap,qsat,dz,modv,d_theta
      REAL, DIMENSION(klev+1) :: kz, Ri, N2, M, diffz
      REAL fq_sat,fcoriolis

      REAL, DIMENSION(klev) :: d_t_dyn,d_q_dyn,d_u_dyn,d_v_dyn
      REAL, SAVE, ALLOCATABLE :: t_pre(:),q_pre(:),u_pre(:),v_pre(:)

      REAL, DIMENSION(klev) :: masse

      REAL, SAVE :: flux_sensible=-10000, flux_latent=-10000
      INTEGER, SAVE :: flag_init=0
      INTEGER, SAVE :: flag_forcing=0,flag_coriolis=0,flag_panache=0

      INTEGER        longcles
      PARAMETER    ( longcles = 20 )

      REAL temp_newton(klev)
      INTEGER k,kinv,ktop
      logical, SAVE :: first=.true.

      REAL taux, tauy
      REAL g, l0, Ric, emin
      REAL lmix(klev), z(klev) 
      REAL clesphy0( longcles      )

      REAL, dimension(klev) :: thetath, buoy,ent,det

      REAL, dimension(klev+1) :: flux,vitw,vitw2,Ftheta

      REAL alpha, epsi


!=====================================================================
! 1/ Some initialisations
!=====================================================================

if ( nlon /= 1 ) then
    stop'physiq1dnum est fait pour tourner en 1D'
endif

print*,'CORIOLIS ',fcoriolis
print*,'ENTREE DANS PHYSIQ'

ug=10.
vg=0.
g=9.81
emin=10**(-12)
l0=20
Ric=0.25

if (first) then
     allocate (t_pre(klev),q_pre(klev),u_pre(klev),v_pre(klev))
     d_u_dyn=0. ; d_v_dyn=0. ; d_t_dyn=0. ; d_q_dyn=0.
     call iophys_ini('phys.nc    ')
     first=.false.

! Lecture de parametres de controle des simulations
     call getin('flux_sensible',flux_sensible)
     call getin('flux_latent',flux_latent)
     call getin('flag_init',flag_init)
     call getin('flag_panache',flag_panache)
     call getin('flag_forcing',flag_forcing)
     call getin('flag_coriolis',flag_coriolis)
     if (flag_init==1) then
     ! On met l'eau et les traceurs à zero au debut
        qx=-d_qx/pdtphys
        qx=0.
     else if (flag_init==2) then
     ! On met l'eau et les traceurs à zero au debut
        qx=-d_qx/pdtphys
        qx=0.
     ! On prend une temperature potentielle uniforme theta=300K
        do k=1,klev
           temp(k)=300.*(paprs(k)/paprs(1))**(rd/rcpd)
        enddo


     endif
else

! Si flag_forcing>0, on elimine le forçage du cas 1D pour le redéfinir
! en direct.
     if (flag_forcing>0) then ! on reprend les valeurs du pas precedent
        vitu(:)=u_pre(:)
        vitv(:)=v_pre(:)
        temp(:)=t_pre(:)
        qx(:,1)=q_pre(:)
     endif
     d_u_dyn(:)=(vitu(:)-u_pre(:))/pdtphys
     d_v_dyn(:)=(vitv(:)-v_pre(:))/pdtphys
     d_t_dyn(:)=(temp(:)-t_pre(:))/pdtphys
     d_q_dyn(:)=(qx(:,1)-q_pre(:))/pdtphys
endif

d_u=0.
d_v=0.
d_t=0.
d_qx=0.
d_ps=0.


!=====================================================================
! 2/ Some usefull cocmputations
!=====================================================================
! Variables at mid-layer
!----------------------------------------------------

fcoriolis=2.*sin(rpi/4.)*2*rpi/86400.
do k=1,klev
   ovap(k)=qx(k,1)
   rho(k)=pplay(k)/(rd*temp(k))
   qsat(k)=fq_sat(temp(k),pplay(k)/100.)
   theta(k)=temp(k)*(paprs(1)/paprs(k))**(rd/rcpd)
   rhodz(k)=(paprs(k)-paprs(k+1))/rg
   dz(k)=rhodz(k)/rho(k)
   modv(k)=vitu(k)*vitu(k)+vitv(k)*vitv(k)
   masse(k)=(paprs(k)-paprs(k+1))/RG
   z(k)=pphi(k)/g
enddo


!=====================================================================
! 3/ Vertical turbulent diffusion
!=====================================================================

! Calcul du coefficient de diffusion turbulente
call calculkz(klev,pphi,theta,vitv,vitu,kz)


! Surface fluxes, sensible and latent heat
if ( flux_latent > -9999. ) flat=-flux_latent
if ( flux_sensible > -9999. ) fsens=-flux_sensible
print*,'fsens=',fsens
print*,'flat=',flat

! Vertical diffusion computation
taux=-vitu(1)/100.
tauy=-vitv(1)/100.

CALL diffkz(klev,nqtot,rho,rhodz,kz,theta,vitu,vitv,qx, &
   &         taux,tauy,fsens,flat,d_theta,d_u,d_v,d_qx)

if(flag_coriolis==1) then
d_u(:)=d_u(:)+fcoriolis*(vitv(:)-vg)
d_v(:)=d_v(:)-fcoriolis*(vitu(:)-ug)
endif

! Changing potential temperature tendency to natural
d_t(:)=d_theta(:)*temp(:)/theta(:)

vitw=0.
if (flag_panache==1) then
call panache(rg,temp,theta,rho,rhodz,klev,dz,thetath,Ftheta,flux,ent,det,vitw,d_t)
endif


!============================ call iophys_ecrit('kz',llm,'Kz','m2/s',kz)================================================
! 9/ Writing outputs
!============================================================================

print*,'PHY1DNUM OK5'
 call iophys_ecrit('w',llm,'w','m/s',vitw)
 call iophys_ecrit('kz',llm,'Kz','m2/s',kz)
 call iophys_ecrit('kinv',1,'Kinv',' ',float(kinv))
 call iophys_ecrit('dz',llm,'dz','m',dz)
 call iophys_ecrit('u',llm,'Vent zonal moy','m/s',vitu)
 call iophys_ecrit('v',llm,'Vent meridien moy','m/s',vitv)
 call iophys_ecrit('temp',llm,'Temperature','K',temp)
 call iophys_ecrit('ovap',llm,'humidite specifique ','g/kg',ovap)
 call iophys_ecrit('rhum',llm,'humidite relative ','g/kg',ovap/qsat)
 call iophys_ecrit('qsat',llm,'humidite a saturation ','g/kg',qsat)
 call iophys_ecrit('theta',llm,'Temperature pot','K',theta)
 call iophys_ecrit('geop',llm,'Geopotential','m2/s2',pphi)
 call iophys_ecrit('masse',llm,'layer masse','kg/m2',masse)
 call iophys_ecrit('ps',1,'Surface pressure','Pa',paprs)
 call iophys_ecrit('sens',1,'Surface sensible flux','W/m2',fsens)
 call iophys_ecrit('flat',1,'Surface latent flux','W/m2',flat)

 call iophys_ecrit('d_u',llm,'Tendance u','m/s/s',d_u)
 call iophys_ecrit('d_v',llm,'Tendance v','m/s/s',d_v)
 call iophys_ecrit('d_t',llm,'Tendance t','K/s',d_t)
 call iophys_ecrit('d_qx',llm,'Tendance vap','kg/kg/s',d_qx)

 call iophys_ecrit('d_u_dyn',llm,'d_u_dyn','m/s',d_u_dyn)
 call iophys_ecrit('d_v_dyn',llm,'d_v_dyn','m/s',d_v_dyn)
 call iophys_ecrit('d_t_dyn',llm,'d_t_dyn','m/s',d_t_dyn)
 call iophys_ecrit('d_q_dyn',llm,'d_q_dyn','m/s',d_q_dyn)

      print*,'COUCOU B PHYDEV'
u_pre(:)=vitu(:)+d_u(:)*pdtphys
v_pre(:)=vitv(:)+d_v(:)*pdtphys
t_pre(:)=temp(:)+d_t(:)*pdtphys
q_pre(:)=qx(:,1)+d_qx(:,1)*pdtphys

      return
      end

