! $Id: physiq.F 1565 2011-08-31 12:53:29Z jghattas $
!#define IO_DEBUG

      SUBROUTINE phys1dThomas(nlon,nlev, &
     &            debut,lafin,pdtphys, &
     &            paprs,pplay,pphi,pphis,presnivs,clesphy0, &
     &            vitu,vitv,temp,qx, &
     &            flxmass_w, &
     &            d_u, d_v, d_t, d_qx, d_ps &
     &            , dudyn &
     &            , PVteta)

      USE dimphy
      USE infotrac
      USE geometry_mod, ONLY: cell_area, latitude_deg, longitude_deg
      USE phys_cal_mod, only: year_len, mth_len, days_elapsed, jh_1jan, &
         year_cur, mth_cur,jD_cur,jH_cur, jD_ref
      USE ioipsl , ONLY : getin

      IMPLICIT NONE
!======================================================================
! Objet: Moniteur general de la physique du modele
!======================================================================
!
!  Arguments:
!
! nlon----input-I-nombre de points horizontaux
! nlev----input-I-nombre de couches verticales, doit etre egale a klev
! debut---input-L-variable logique indiquant le premier passage
! lafin---input-L-variable logique indiquant le dernier passage
! jD_cur       -R-jour courant a l'appel de la physique (jour julien)
! jH_cur       -R-heure courante a l'appel de la physique (jour julien)
! pdtphys-input-R-pas d'integration pour la physique (seconde)
! paprs---input-R-pression pour chaque inter-couche (en Pa)
! pplay---input-R-pression pour le mileu de chaque couche (en Pa)
! pphi----input-R-geopotentiel de chaque couche (g z) (reference sol)
! pphis---input-R-geopotentiel du sol
! presnivs-input_R_pressions approximat. des milieux couches ( en PA)
! vitu-------input-R-vitesse dans la direction X (de O a E) en m/s
! vitv-------input-R-vitesse Y (de S a N) en m/s
! temp-------input-R-temperature (K)
! qx------input-R-humidite specifique (kg/kg) et d'autres traceurs
! d_t_dyn-input-R-tendance dynamique pour "t" (K/s)
! d_q_dyn-input-R-tendance dynamique pour "q" (kg/kg/s)
! flxmass_w -input-R- flux de masse verticale
! d_u-----output-R-tendance physique de "u" (m/s/s)
! d_v-----output-R-tendance physique de "v" (m/s/s)
! d_t-----output-R-tendance physique de "t" (K/s)
! d_qx----output-R-tendance physique de "qx" (kg/kg/s)
! d_ps----output-R-tendance physique de la pression au sol
!IM
! PVteta--output-R-vorticite potentielle a des thetas constantes
!======================================================================

!============================================================================
! 0/ Declarations
!============================================================================

! include
#include "dimensions.h"
#include "YOMCST.h"
#include "flux_arp.h"


      INTEGER jjmp1
      parameter (jjmp1=jjm+1-1/jjm)
      INTEGER iip1
      parameter (iip1=iim+1)

      INTEGER ivap          ! indice de traceurs pour vapeur d'eau
      PARAMETER (ivap=1)
      INTEGER iliq          ! indice de traceurs pour eau liquide
      PARAMETER (iliq=2)

! Variables argument:
      INTEGER nlon
      INTEGER nlev

      REAL pdtphys
      LOGICAL debut, lafin
      REAL paprs(klev+1)
      REAL pplay(klev)
      REAL pphi(klev)
      REAL pphis
      REAL presnivs(klev)
      REAL znivsig(klev)

      REAL vitu(klev)
      REAL vitv(klev)
      REAL temp(klev),theta(klev)
      REAL qx(klev,nqtot)
      REAL flxmass_w(klev)
      REAL omega(klev) ! vitesse verticale en Pa/s
      REAL d_u(klev)
      REAL d_v(klev)
      REAL d_t(klev)
      REAL d_qx(klev,nqtot)
      REAL d_ps
      REAL da(klev),phi(klev,klev),mp(klev)
      INTEGER nbteta
      PARAMETER(nbteta=3)
      REAL PVteta(nbteta)

      REAL dudyn(iip1,jjmp1,klev)
      REAL, DIMENSION(klev) :: rho,rhodz,ovap,qsat,dz,modv,d_theta
      REAL, DIMENSION(klev+1) :: kz,ri
      REAL fq_sat,fcoriolis

      REAL, DIMENSION(klev) :: d_t_dyn,d_q_dyn,d_u_dyn,d_v_dyn
      REAL, SAVE, ALLOCATABLE :: t_pre(:),q_pre(:),u_pre(:),v_pre(:)

      REAL, DIMENSION(klev) :: masse

      REAL, SAVE :: flux_sensible=-10000., flux_latent=-10000.
      INTEGER, SAVE :: flag_init=0, flag_forcing=1

      INTEGER        longcles
      PARAMETER    ( longcles = 20 )

      REAL temp_newton(klev)
      INTEGER k,kinv,ktop
      logical, SAVE :: first=.true.

      REAL taux, tauy

      REAL clesphy0( longcles      )

! NEW declarations IRENE

REAL, save :: Ts
REAL LWdown,LWup,SW,Cd

! NEW declarations THOMAS

      REAL vitw(klev)
      REAL wsub(klev)
      REAL kappaz(klev)
      REAL rapp(klev)
      REAL frac_nuag(klev)
      ! pour switch entre kz (=/=1) et kappaz (=1)
      INTEGER, SAVE ::  choix_nuage=1, choix_kz=1
      REAL, SAVE :: wsub0=0.01,lkap=10.


!=====================================================================
! 1/ Some initialisations
!=====================================================================

if ( nlon /= 1 ) then
    stop'physiq1dnum est fait pour tourner en 1D'
endif

print*,'CORIOLIS ',fcoriolis
print*,'ENTREE DANS PHYSIQ'

if (first) then
     allocate (t_pre(klev),q_pre(klev),u_pre(klev),v_pre(klev))
     d_u_dyn=0. ; d_v_dyn=0. ; d_t_dyn=0. ; d_q_dyn=0.
     call iophys_ini('phys.nc    ')
     first=.false.

! Lecture de parametres de controle des simulations
     call getin('wsub0',wsub0)
     call getin('lkap',lkap)
     call getin('choix_nuage',choix_nuage)
     call getin('choix_kz',choix_kz)
     call getin('flux_sensible',flux_sensible)
     call getin('flux_latent',flux_latent)
     call getin('flag_init',flag_init)
     call getin('flag_forcing',flag_forcing)

     if (flag_init==1) then
     ! On met l'eau et les traceurs à zero au debut
        qx=-d_qx/pdtphys
        qx=0.
     else if (flag_init==2) then
     ! On met l'eau et les traceurs à zero au debut
        qx=-d_qx/pdtphys
        qx=0.
     ! On prend une temperature potentielle uniforme theta=300K
        do k=1,klev
           temp(k)=300.*(paprs(k)/paprs(1))**(rd/rcpd)
        enddo


     endif
else

! Si flag_forcing>0, on elimine le forçage du cas 1D pour le redéfinir
! en direct.
     if (flag_forcing>0) then ! on reprend les valeurs du pas precedent
        vitu(:)=u_pre(:)
        vitv(:)=v_pre(:)
        temp(:)=t_pre(:)
        qx(:,1)=q_pre(:)
     endif
     d_u_dyn(:)=(vitu(:)-u_pre(:))/pdtphys
     d_v_dyn(:)=(vitv(:)-v_pre(:))/pdtphys
     d_t_dyn(:)=(temp(:)-t_pre(:))/pdtphys
     d_q_dyn(:)=(qx(:,1)-q_pre(:))/pdtphys
endif

d_u=0.
d_v=0.
d_t=0.
d_qx=0.
d_ps=0.


!=====================================================================
! 2/ Some usefull cocmputations
!=====================================================================
! Variables at mid-layer
!----------------------------------------------------

fcoriolis=2.*sin(rpi/4.)*2*rpi/86400.
do k=1,klev
   ovap(k)=qx(k,1)
   rho(k)=pplay(k)/(rd*temp(k))
   qsat(k)=fq_sat(temp(k),pplay(k)/100.)
   theta(k)=temp(k)*(paprs(1)/paprs(k))**(rd/rcpd)
   rhodz(k)=(paprs(k)-paprs(k+1))/rg
   dz(k)=rhodz(k)/rho(k)
   modv(k)=vitu(k)*vitu(k)+vitv(k)*vitv(k)
   masse(k)=(paprs(k)-paprs(k+1))/RG
enddo


! NEW calculus THOMAS

vitw(:)=0.
do k=2,klev
   vitw(k)=vitw(k-1)**2+2*rg*dz(k)*(theta(1)-theta(k))/theta(k)
   if (vitw(k)<0) then
      vitw(k)=0
      exit
   endif
   vitw(k)=sqrt(vitw(k))
enddo
kappaz(:)=lkap*max(vitw(:),0.001*(max(1000-pphi(:)/RG,0.)))!/dz(:)**2
! On met une valeur minimum de kz
kappaz(2)=kappaz(3)/2.
if (choix_nuage==0) then
   rapp(:)=0.2
elseif (choix_nuage==1) then
   do k=1,klev
      rapp(k)=0.2*pphi(k)/rg/1000.
   enddo
elseif (choix_nuage==2) then
   do k=1,klev
      rapp(k)=abs(ovap(1)-ovap(k))/ovap(k)
   enddo
endif
do k=1,klev
   if (qsat(k)<ovap(k)*(1-rapp(k)/2)) then
      frac_nuag(k)=1
   elseif (qsat(k)>ovap(k)*(1+rapp(k)/2)) then
      frac_nuag(k)=0
   else
      frac_nuag(k)=(ovap(k)+0.5*rapp(k)*ovap(k)-qsat(k))/(rapp(k)*ovap(k))
   endif
enddo

!do k=3,klev
!   d_qx(k,:)=kappaz(k)*(qx(k,:)-2*qx(k-1,:)+qx(k-2,:))
!enddo



!=====================================================================
! 3/ Vertical turbulent diffusion
!=====================================================================

! Calcul du coefficient de diffusion turbulente
do k=1,klev
   kz(k)=2.*(1.-tanh(2.*(k-klev*5./6.)))/2.
enddo

! Surface fluxes, sensible and latent heat
if ( flux_latent > -9999. ) flat=-flux_latent
if ( flux_sensible > -9999. ) fsens=-flux_sensible
print*,'fsens=',fsens
print*,'flat=',flat

! Vertical diffusion computation
taux=-vitu(1)/100.
tauy=-vitv(1)/100.

! NEW modification IRENE

 CALL fluxsurf(jh_cur,pdtphys,vitu(1),vitv(1),qx(1,1), &
   & pphi(1)/RG,paprs(1),temp(1), flat, fsens, sw, lwdown, lwup, &
   & taux, tauy,Cd, Ts)

! NEW modification THOMAS

if (choix_kz==1) then
CALL diffkz(klev,nqtot,rho,rhodz,kappaz,theta,vitu,vitv,qx, &
   &         taux,tauy,fsens,flat,d_theta,d_u,d_v,d_qx)
else
CALL diffkz(klev,nqtot,rho,rhodz,kz,theta,vitu,vitv,qx, &
   &         taux,tauy,fsens,flat,d_theta,d_u,d_v,d_qx)
endif
d_t(:)=d_theta(:)*temp(:)/theta(:)

if (wsub0>0.) then
   do k=1,klev
      wsub(k)=wsub0*sin(rpi*(pplay(k)-pplay(klev))/(paprs(1)-pplay(klev)))   
   enddo
   do k=1,klev-1
      d_qx(k,1)=d_qx(k,1)+wsub(k)*rg*(ovap(k+1)-ovap(k))/(pphi(k+1)-pphi(k))
      d_t(k)=d_t(k)+temp(k)/theta(k) * &
    &      wsub(k)*rg*(theta(k+1)-theta(k))/(pphi(k+1)-pphi(k))
   enddo
endif

! END modification THOMAS

! Changing potential temperature tendency to natural


!============================================================================
! 9/ Writing outputs
!============================================================================

print*,'PHY1DNUM OK5'
 call iophys_ecrit('kz',llm,'Kz','m2/s',kz)
 call iophys_ecrit('kinv',1,'Kinv',' ',float(kinv))
 call iophys_ecrit('dz',llm,'dz','m',dz)
 call iophys_ecrit('u',llm,'Vent zonal moy','m/s',vitu)
 call iophys_ecrit('v',llm,'Vent meridien moy','m/s',vitv)
 call iophys_ecrit('temp',llm,'Temperature','K',temp)
 call iophys_ecrit('ovap',llm,'humidite specifique ','g/kg',ovap)
 call iophys_ecrit('rhum',llm,'humidite relative ','g/kg',ovap/qsat)
 call iophys_ecrit('qsat',llm,'humidite a saturation ','g/kg',qsat)
 call iophys_ecrit('theta',llm,'Temperature pot','K',theta)
 call iophys_ecrit('geop',llm,'Geopotential','m2/s2',pphi)
 call iophys_ecrit('masse',llm,'layer masse','kg/m2',masse)
 call iophys_ecrit('ps',1,'Surface pressure','Pa',paprs)
 call iophys_ecrit('sens',1,'Surface sensible flux','W/m2',fsens)
 call iophys_ecrit('flat',1,'Surface latent flux','W/m2',flat)

 call iophys_ecrit('d_u',llm,'Tendance u','m/s/s',d_u)
 call iophys_ecrit('d_v',llm,'Tendance v','m/s/s',d_v)
 call iophys_ecrit('d_t',llm,'Tendance t','K/s',d_t)
 call iophys_ecrit('d_qx',llm,'Tendance vap','kg/kg/s',d_qx)

 call iophys_ecrit('d_u_dyn',llm,'d_u_dyn','m/s',d_u_dyn)
 call iophys_ecrit('d_v_dyn',llm,'d_v_dyn','m/s',d_v_dyn)
 call iophys_ecrit('d_t_dyn',llm,'d_t_dyn','m/s',d_t_dyn)
 call iophys_ecrit('d_q_dyn',llm,'d_q_dyn','m/s',d_q_dyn)

! NEW writting calculus IRENE

 call iophys_ecrit('LWdown',1,'LWdown','W/m2',LWdown)
 call iophys_ecrit('LWup',1,'LWup','W/m2',LWup)
 call iophys_ecrit('SW',1,'SW','W/m2',SW)

! NEW writting calculus THOMAS

 call iophys_ecrit('wasc',llm,'Vent vertical ascendant','m/s',vitw)
 call iophys_ecrit('wsub',llm,'Vent vertical subsident','m/s',wsub)
 call iophys_ecrit('w',llm,'Vent vertical total','m/s',wsub+vitw)
 call iophys_ecrit('kappaz',llm,'Coef turbulent/dz2','/s',kappaz)
 call iophys_ecrit('rapp',llm,'rapport q/sigma','S.U.',rapp)
 call iophys_ecrit('fracnuag',llm,'fraction nuageuse','S.U.',frac_nuag)

      print*,'COUCOU B PHYDEV'
u_pre(:)=vitu(:)+d_u(:)*pdtphys
v_pre(:)=vitv(:)+d_v(:)*pdtphys
t_pre(:)=temp(:)+d_t(:)*pdtphys
q_pre(:)=qx(:,1)+d_qx(:,1)*pdtphys

      return
      end

